<?php
/**
 * Created by PhpStorm.
 * User: gaupoit
 * Date: 7/13/18
 * Time: 16:45
 */

use W3TC\Dispatcher;

if ( ! function_exists( 'is_plugin_active' ) ) {
	include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
}

/**
 * Check whether is gold function
 * @return bool
 */
function wpp_is_gold_function() {
	return is_dir( PDA_ADMIN_GOLD_PATH );
}

/**
 * Check license
 * @return bool
 */
function wpp_is_entered_license() {
	return get_option( WP_Protect_Password_Constant::LICENSE_OPTIONS ) === "1" ? true : false;
}

/**
 * Clean all data when delete plugin
 */


/**
 * Delete all data in postmeta and options
 */
function delete_pda_password_options() {
	if ( wpp_is_gold_function() ) {
		$ppwp_remove_data = wpp_get_settings_boolean( WP_Protect_Password_Constant::WPP_REMOVE_DATA );
		if ( $ppwp_remove_data ) {
			delete_option( WP_Protect_Password_Constant::GENERAL_SET_PASSWORD_OPTIONS );
			delete_option( WP_Protect_Password_Constant::GENERAL_SETTING_OPTIONS );
			delete_option( WP_Protect_Password_Constant::TBL_VERSION );
			delete_option( WP_Protect_Password_Constant::LICENSE_KEY );
			delete_option( WP_Protect_Password_Constant::LICENSE_ERROR );
			delete_option( WP_Protect_Password_Constant::LICENSE_OPTIONS );
			delete_data_in_post_meta_by_meta_key( WP_Protect_Password_Constant::POST_PROTECTION_ROLES );
			delete_data_in_post_meta_by_meta_key( WP_Protect_Password_Constant::AUTO_GENERATE_PWD_META_DATA );
			WP_Protect_Password_DB::uninstall();
		}
	} else {
		delete_option( WP_Protect_Password_Constant::GENERAL_SET_PASSWORD_OPTIONS );
		delete_option( WP_Protect_Password_Constant::GENERAL_SETTING_OPTIONS );
		delete_data_in_post_meta_by_meta_key( WP_Protect_Password_Constant::POST_PROTECTION_ROLES );
		delete_data_in_post_meta_by_meta_key( WP_Protect_Password_Constant::MULTIPLE_PASSWORDS_POST_META );
	}
}

/**
 * Clean data in postmeta
 */
function delete_data_in_post_meta_by_meta_key( $meta_key ) {
	global $wpdb;
	$table_postmeta = $wpdb->prefix . "postmeta";
	$wpdb->delete( $table_postmeta, array(
		'meta_key' => $meta_key,
	) );
}

function wp_protect_password_get_license_type() {
	$app_id = get_site_option( WP_Protect_Password_Constant::APP_ID, null );
	if ( is_null( $app_id ) ) {
		return "N/A";
	}
	switch ( $app_id ) {
		case "583147":
			return "1-site license";
			break;
		case "584087":
			return "3-site license";
			break;
		case "77844608":
			return "15-site license";
			break;
		case "584088":
			return "Unlimited-site license";
			break;
		case "77814469":
			return "Developer license";
			break;
		case "77808414":
			return "3-site license";
			break;
		case "78022875":
			return "3-site lifetime license";
			break;
		default:
			return "N/A";
			break;
	}
}

/**
 * Get setting type is array
 *
 * prams : $name_settings
 *
 * return array
 */
function wpp_entire_site_get_setting_type_is_array( $name_settings ) {
	$settings = get_option( WP_Protect_Password_Constant::GENERAL_SET_PASSWORD_OPTIONS, false );
	if ( $settings ) {
		$options = @unserialize( $settings );
		if ( false === $options || empty( $options ) ) {
			return array();
		}
		if ( array_key_exists( $name_settings, $options ) && ! empty( $options[ $name_settings ] ) ) {
			return $options[ $name_settings ];
		}
	}

	return array();
}

/**
 * Get setting type is string
 *
 * @param $name_settings
 *
 * @return string
 */
function wpp_entire_site_get_setting_type_is_string( $name_settings ) {
	$settings = get_option( WP_Protect_Password_Constant::GENERAL_SET_PASSWORD_OPTIONS, false );
	if ( $settings ) {
		$options = @unserialize( $settings );
		if ( false === $options || empty( $options ) ) {
			return '';
		}
		if ( array_key_exists( $name_settings, $options ) && ! empty( $options[ $name_settings ] ) ) {
			return $options[ $name_settings ];
		}
	}

	return '';
}

/**
 * Get setting type is string
 *
 * @param $name_settings
 *
 * @return bool
 */
function wpp_entire_site_get_setting_type_is_bool( $name_settings ) {
	$settings = get_option( WP_Protect_Password_Constant::GENERAL_SET_PASSWORD_OPTIONS, false );
	if ( $settings ) {
		$options = @unserialize( $settings );
		if ( false === $options || empty( $options ) ) {
			return false;
		}
		if ( array_key_exists( $name_settings, $options ) && 'true' === $options[ $name_settings ] ) {
			return true;
		}
	}

	return false;
}

/**
 * @param $name_settings
 *
 * @return array|string
 */
function wpp_get_settings_value( $name_settings ) {
	$settings = get_option( WP_Protect_Password_Constant::GENERAL_SETTING_OPTIONS );
	if ( empty( $settings ) ) {
		return array();
	}
	$options = json_decode( $settings );
	if ( ! isset( $options->$name_settings ) || $options->$name_settings === '' ) {
		return array();
	}

	return $options->$name_settings;
}

/**
 * @param $name_settings
 *
 * @return bool
 */
function wpp_get_site_settings_boolean( $name_settings ) {
	$settings = get_site_option( WP_Protect_Password_Constant::GENERAL_SETTING_OPTIONS );

	return wpp_check_option_settings( $settings, $name_settings );
}

/**
 * @param $name_settings
 *
 * @return bool
 */
function wpp_get_settings_boolean( $name_settings ) {
	$settings = get_option( WP_Protect_Password_Constant::GENERAL_SETTING_OPTIONS );

	return wpp_check_option_settings( $settings, $name_settings );
}

/**
 * @param $settings
 * @param $name_settings
 *
 * @return bool
 */
function wpp_check_option_settings( $settings, $name_settings ) {
	if ( $settings ) {
		$options = json_decode( $settings );

		if ( ! isset( $options->$name_settings ) ) {
			return false;
		}

		if ( $options->$name_settings == "true" || $options->$name_settings == "1" ) {
			return true;
		}
	}

	return false;
}

/**
 * TODO: Add this function to plugin-update class.
 * Set default settings values
 * @return bool
 */
function ppwp_set_default_settings() {
	$setting                                 = get_option( WP_Protect_Password_Constant::GENERAL_SETTING_OPTIONS );
	$auto_protect_all_child_pages            = WP_Protect_Password_Constant::WPP_AUTO_PROTECT_ALL_CHILD_PAGES;
	$ppwp_remove_search_engine               = WP_Protect_Password_Constant::WPP_REMOVE_SEARCH_ENGINE;
	$ppwp_whitelist_column_protections       = WP_Protect_Password_Constant::WPP_WHITELIST_COLUMN_PROTECTIONS;
	$value_ppwp_whitelist_column_protections = ppwp_get_default_options_via_other_plugin( $ppwp_whitelist_column_protections, WP_Protect_Password_Constant::WOOCOMMERCE_PLUGIN_NAME );

	if ( ! $setting ) {
		$ppwp_whitelist_role  = WP_Protect_Password_Constant::WPP_WHITELIST_ROLES;
		$ppwp_setting_default = array(
			$auto_protect_all_child_pages      => "true",
			$ppwp_remove_search_engine         => "true",
			$ppwp_whitelist_role               => "admin_users",
			$ppwp_whitelist_column_protections => $value_ppwp_whitelist_column_protections
		);

		return update_option( WP_Protect_Password_Constant::GENERAL_SETTING_OPTIONS, wp_json_encode( $ppwp_setting_default ) );
	}
	$options = json_decode( $setting );
	check_setting_existed_to_set_default_value( $options, $auto_protect_all_child_pages, "true" );
	check_setting_existed_to_set_default_value( $options, $ppwp_remove_search_engine, "true" );
	check_setting_existed_to_set_default_value( $options, $ppwp_whitelist_column_protections, $value_ppwp_whitelist_column_protections );

	return false;
}

/**
 * @param $name_option
 * @param $name_plugin
 *
 * @return mixed
 */
function ppwp_get_default_options_via_other_plugin( $name_option, $name_plugin ) {
	if ( ! ppwp_is_plugin_active( WP_Protect_Password_Constant::PLUGINS_PATH[ $name_plugin ] ) ) {
		return WP_Protect_Password_Constant::DEFAULT_VALUE_SETTINGS[ $name_option ];
	}
	$default_with_plugin = WP_Protect_Password_Constant::DEFAULT_VALUE_SETTINGS[ $name_plugin ];

	return $default_with_plugin[ $name_option ];
}

function check_setting_existed_to_set_default_value( $options, $setting_key, $value ) {
	if ( isset( $options->$setting_key ) ) {
		return false;
	}
	$options->$setting_key = $value;

	return update_option( WP_Protect_Password_Constant::GENERAL_SETTING_OPTIONS, wp_json_encode( $options ) );
}

/**
 * Get all posts password protected by WordPress
 *
 * @param array $posts_type
 *
 * @return mixed
 */
function ppwp_get_posts_password_protected_by_wp( $posts_type = array( 'page', 'post' ) ) {
	$results = get_posts( array(
		'post_status'  => 'publish',
		'post_type'    => $posts_type,
		'numberposts'  => - 1,
		'has_password' => true,
	) );

	return $results;
}

/**
 * Migrate default password and update post password of Wordpress for free version
 */
function migrate_default_password_to_ppwp() {
	$posts = ppwp_get_posts_password_protected_by_wp();
	foreach ( $posts as $post ) {
		$global_password = get_post_meta( $post->ID, WP_Protect_Password_Constant::MULTIPLE_PASSWORDS_POST_META, true );
		$global_password = ! empty( $global_password ) ? $global_password : array();
		$role_password   = get_post_meta( $post->ID, WP_Protect_Password_Constant::POST_PROTECTION_ROLES, true );

		// 1. Update password for role
		if ( ! empty( $role_password ) ) {
			$role_password = wp_unslash( unserialize( $role_password ) );
			foreach ( $role_password as $key => $value ) {
				if ( str_replace( " ", "", $post->post_password ) === $value ) {
					$role_password[ $key ] = '';
					update_post_meta( $post->ID, WP_Protect_Password_Constant::POST_PROTECTION_ROLES, serialize( $role_password ) );
				}
			}
		}

		// 2. Update password for global
		if ( ! in_array( str_replace( " ", "", $post->post_password ), $global_password ) ) {
			array_push( $global_password, str_replace( " ", "", $post->post_password ) );
		}
		update_post_meta( $post->ID, WP_Protect_Password_Constant::MULTIPLE_PASSWORDS_POST_META, wp_slash( $global_password ) );

		// 3. Update default password for Wordpress
		wp_update_post( array(
			'ID'            => $post->ID,
			'post_password' => '',
		) );
	}
}

/**
 * @param string $meta_key Meta key
 *
 * @return array Array post meta
 */
function get_post_meta_by_meta_key( $meta_key ) {
	global $wpdb;

	return $value = $wpdb->get_results( $wpdb->prepare( "SELECT post_id, meta_value  FROM $wpdb->postmeta WHERE meta_key = %s", $meta_key ) );
}

/**
 * Validate global password
 *
 * @param $multiple_passwords
 * @param $role_password
 *
 * @return bool
 */
function free_validate_duplicate_global_password( $multiple_passwords, $role_password ) {
	if ( empty( $role_password ) ) {
		return false;
	}
	$password_duplicate = array_intersect( $multiple_passwords, array_values( $role_password ) );
	if ( empty( $password_duplicate ) ) {
		return false;
	} else {
		return true;
	}
}

/**
 * Validate role password
 *
 * @param $role_password
 * @param $global_password
 *
 * @return bool
 */
function free_validate_duplicate_role_password( $role_password, $global_password ) {
	if ( empty( $global_password ) ) {
		return false;
	}
	$role_password = wp_unslash( $role_password );
	if ( in_array( $role_password, $global_password ) ) {
		return true;
	} else {
		return false;
	}
}

/**
 * Validate password type is role for free version
 *
 * @param $role
 * @param $password
 * @param $global_password
 * @param $current
 */
function free_validate_password_type_is_role( $role, $password, $global_password, $current ) {
	if ( empty( $password ) ) {
		if ( empty( $current[ $role ] ) ) {
			wp_send_json(
				array(
					'is_error' => true,
					'message'  => 'Please fill out empty fields.',
				)
			);
			wp_die();
		}
	}

	$role_validate = free_validate_duplicate_role_password( $password, $global_password );
	if ( $role_validate ) {
		wp_send_json(
			array(
				'is_error' => true,
				'message'  => 'You can\'t create duplicate password. Please try again.',
			)
		);
		wp_die();
	}
}

/**
 * Validate password type is global for free version
 *
 * @param $ppwp_multiple_password
 * @param $global_password
 * @param $current
 */
function free_validate_password_type_is_global( $ppwp_multiple_password, $global_password, $current ) {
	if ( empty( $ppwp_multiple_password ) ) {
		if ( empty( $global_password ) ) {
			wp_send_json(
				array(
					'is_error' => true,
					'message'  => 'Please fill out empty fields.',
				)
			);
			wp_die();
		}
	}

	$multiple_passwords = array_filter( array_map( 'trim', explode( "\n", $ppwp_multiple_password ) ) );
	$global_validate    = free_validate_duplicate_global_password( $multiple_passwords, $current );
	if ( $global_validate ) {
		wp_send_json(
			array(
				'is_error' => true,
				'message'  => 'You can\'t create duplicate password. Please try again.',
			)
		);
		wp_die();
	}
}

/**
 * Add page or post to list not cache of WP Super Cache plugin for free version  with type is global
 *
 * @param $multiple_passwords
 * @param $id
 * @param $current
 */
function free_add_page_or_post_to_list_not_cache_with_type_is_global( $multiple_passwords, $id, $current ) {
	if ( is_plugin_active( 'wp-super-cache/wp-cache.php' ) ) {
		if ( ! empty( $multiple_passwords ) ) {
			$uri = ppwp_get_uri_by_post_id( $id );
			global $cache_rejected_uri;
			if ( ! in_array( '/' . $uri, $cache_rejected_uri ) ) {
				ppwp_add_page_or_post_to_list_not_cache( $id, $uri, $cache_rejected_uri );
			}
		} else {
			$result = array_values( $current );
			$result = array_filter( $result, 'strlen' );
			if ( empty( $result ) ) {
				ppwp_remove_page_or_post_in_list_not_cache( $id );
			}
		}
	}
}

/**
 * Add page or post to list not cache of WP Super Cache plugin for free version with type is role
 *
 * @param $password
 * @param $id
 * @param $current
 */
function free_add_page_or_post_to_list_not_cache_with_type_is_role( $password, $id, $current ) {
	if ( is_plugin_active( 'wp-super-cache/wp-cache.php' ) ) {
		if ( ! empty( $password ) ) {
			$uri = ppwp_get_uri_by_post_id( $id );
			global $cache_rejected_uri;
			if ( ! in_array( '/' . $uri, $cache_rejected_uri ) ) {
				ppwp_add_page_or_post_to_list_not_cache( $id, $uri, $cache_rejected_uri );
			}
		} else {
			if ( empty( $global_password ) ) {
				$result = array_values( $current );
				$result = array_filter( $result, 'strlen' );
				if ( empty( $result ) ) {
					ppwp_remove_page_or_post_in_list_not_cache( $id );
				}
			}
		}
	}
}

/**
 * Clear cache all page
 */
function ppwp_clear_cache() {
	if ( wpp_is_gold_function() ) {
		return;
	}

	if ( is_plugin_active( 'wp-super-cache/wp-cache.php' ) ) {
		if ( is_multisite() ) {
			wp_cache_clear_cache( get_current_blog_id() );
		} else {
			wp_cache_clear_cache();
		}
	}
}

/**
 * Add page or post to list not cache of WP Super Cache plugin
 *
 * @param $post_id
 */
function ppwp_add_page_or_post_to_list_not_cache( $post_id, $uri, $cache_rejected_uri ) {
	// 1. Add URI to list not cache
	global $wp_cache_config_file;
	array_push( $cache_rejected_uri, '/' . $uri );
	$all_uri = array_map( function ( $key, $cache ) {
		return " $key => '$cache'";
	}, array_keys( $cache_rejected_uri ), $cache_rejected_uri );
	$all_uri = implode( ',', $all_uri );
	$text    = "array($all_uri )";
	wp_cache_replace_line( '^ *\$cache_rejected_uri', "\$cache_rejected_uri = $text;", $wp_cache_config_file );

	// 3. Clear cache by post_id
	wp_cache_post_edit( $post_id );
}

/**
 * Remove page or post in list not cache of WP Super Cache plugin
 *
 * @param $post_id
 */
function ppwp_remove_page_or_post_in_list_not_cache( $post_id ) {
	// 1. Get URI in permalink by post_id
	$uri = ppwp_get_uri_by_post_id( $post_id );
	$uri = '/' . $uri;

	// 2. Remove URI in list not cache
	global $cache_rejected_uri, $wp_cache_config_file;
	$cache_rejected_uri = array_unique( $cache_rejected_uri );
	if ( ( $key = array_search( $uri, $cache_rejected_uri ) ) !== false ) {
		unset( $cache_rejected_uri[ $key ] );
		$all_uri = array_map( function ( $key, $cache ) {
			return " $key => '$cache'";
		}, array_keys( $cache_rejected_uri ), $cache_rejected_uri );
		$all_uri = implode( ',', $all_uri );
		$text    = "array($all_uri )";
		wp_cache_replace_line( '^ *\$cache_rejected_uri', "\$cache_rejected_uri = $text;", $wp_cache_config_file );
	}
}

/**
 * get URI by post_id
 *
 * @param $post_id
 *
 * @return string
 */
function ppwp_get_uri_by_post_id( $post_id ) {
	$permalink = get_permalink( $post_id );
	$parse_url = parse_url( $permalink );
	$uri       = $parse_url['path'];
	$uri       = explode( '/', $uri );
	$uri       = ! empty( $uri[ count( $uri ) - 1 ] ) ? $uri[ count( $uri ) - 1 ] : $uri[ count( $uri ) - 2 ];

	return $uri;
}

/**
 * Add tag meta with content noindex to header
 */
function ppwp_add_tag_meta_no_index_to_head() {
	if ( is_singular() && wpp_get_settings_boolean( WP_Protect_Password_Constant::WPP_REMOVE_SEARCH_ENGINE ) ) {
		global $post;
		$post_id = $post->ID;
		if ( is_protected_page_post( $post_id ) ) {
			?>
			<meta name="robots" content="noindex,follow"/>
			<?php
		}
	}
}

/**
 * Get all page and post
 *
 * @return array
 */
function ppwp_get_all_page_post() {
	$results    = array();
	$post_types = array( 'page', 'post' );
	if ( is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
		array_push( $post_types, 'product' );
	}
	foreach ( $post_types as $post_type ) {
		$args = array(
			'post_status' => 'publish',
			'post_type'   => $post_type,
			'numberposts' => - 1,
		);
		foreach ( get_posts( $args ) as $post ) {
			$ppwp_auto_protect_all_child_pages = wpp_get_settings_boolean( WP_Protect_Password_Constant::WPP_AUTO_PROTECT_ALL_CHILD_PAGES );
			if ( $ppwp_auto_protect_all_child_pages ) {
				if ( $post->post_parent === 0 ) {
					array_push( $results, $post );
				}
			} else {
				array_push( $results, $post );
			}
		}
	}

	return $results;
}

/**
 * Get all post types
 *
 * @param string $output Value to output
 *
 * @return array Array Post types
 *
 */
function ppwp_get_all_post_types( $output = 'objects' ) {
	$args       = array(
		'public' => true,
	);
	$post_types = get_post_types( $args, $output );
	unset( $post_types['attachment'] );

	return $post_types;
}

/**
 * Check plugin is active
 *
 * @param $plugin
 *
 * @return bool
 */
function ppwp_is_plugin_active( $plugin ) {
	if ( ! function_exists( 'is_plugin_active' ) ) {
		return false;
	}

	return is_plugin_active( $plugin );
}

/**
 * @return string
 */
function ppwp_get_home_url_with_ssl() {
	return is_ssl() ? home_url( '/', 'https' ) : home_url( '/' );
}

/**
 * Check W3 Total Cache has config cookie group
 *
 * @return bool
 */
function ppwp_check_has_config_w3_total_cache() {
	$config = Dispatcher::config();
	$groups = $config->get_array( 'pgcache.cookiegroups.groups' );
	if ( array_key_exists( 'password_protect_wordpress', $groups ) ) {
		if ( $groups['password_protect_wordpress']['enabled'] && ! $groups['password_protect_wordpress']['cache'] ) {
			$cookies = $groups['password_protect_wordpress']['cookies'];
			if ( wpp_is_gold_function() ) {
				if ( in_array( 'pda_protect_password', $cookies ) && in_array( 'wp-goldpass-pagepost_*', $cookies ) ) {
					return true;
				}
			} else {
				if ( in_array( 'pda_protect_password', $cookies ) && in_array( 'wp-postpass-role_*', $cookies ) ) {
					return true;
				}
			}

		}
	}

	return false;
}


/**
 * Update cookie to option and modify htaccess file for WP Fastest Cache plugin
 */
function ppwp_custom_option_exclude_page_and_cookie_wp_fastest_cache() {
	if ( is_plugin_active( 'wp-fastest-cache/wpFastestCache.php' ) ) {
		$old_fastest   = get_option( "WpFastestCacheExclude" );
		$fastest_cache = new WpFastestCache();
		if ( $old_fastest ) {
			$old_fastest = json_decode( $old_fastest );
			$tmp1        = false;
			if ( wpp_is_gold_function() ) {
				$tmp2_gold = false;
				foreach ( $old_fastest as $fast ) {
					if ( 'cookie' === $fast->type && 'contain' === $fast->prefix && 'pda_protect_password' === $fast->content ) {
						$tmp1 = true;
					}
					if ( 'cookie' === $fast->type && 'contain' === $fast->prefix && 'wp-goldpass-pagepost_' === $fast->content ) {
						$tmp2_gold = true;
					}
				}
				if ( ! $tmp1 ) {
					array_push( $old_fastest, (object) array(
						'prefix'  => 'contain',
						'content' => 'pda_protect_password',
						'type'    => 'cookie'
					) );
				}
				if ( ! $tmp2_gold ) {
					array_push( $old_fastest, (object) array(
						'prefix'  => 'contain',
						'content' => 'wp-goldpass-pagepost_',
						'type'    => 'cookie'
					) );
				}
			} else {
				$tmp2_free = false;
				foreach ( $old_fastest as $fast ) {
					if ( 'cookie' === $fast->type && 'contain' === $fast->prefix && 'pda_protect_password' === $fast->content ) {
						$tmp1 = true;
					}
					if ( 'cookie' === $fast->type && 'contain' === $fast->prefix && 'wp-postpass-role_' === $fast->content ) {
						$tmp2_free = true;
					}
				}
				if ( ! $tmp1 ) {
					array_push( $old_fastest, (object) array(
						'prefix'  => 'contain',
						'content' => 'pda_protect_password',
						'type'    => 'cookie'
					) );
				}
				if ( ! $tmp2_free ) {
					array_push( $old_fastest, (object) array(
						'prefix'  => 'contain',
						'content' => 'wp-goldpass-pagepost_',
						'type'    => 'cookie'
					) );
				}
			}
			update_option( "WpFastestCacheExclude", json_encode( $old_fastest ) );
			$fastest_cache->modify_htaccess_for_exclude();
		} else {
			$ppwp_cookie = array(
				(object) array(
					'prefix'  => 'contain',
					'content' => 'pda_protect_password',
					'type'    => 'cookie'
				)
			);
			if ( wpp_is_gold_function() ) {
				array_push( $ppwp_cookie, (object) array(
					'prefix'  => 'contain',
					'content' => 'wp-goldpass-pagepost_',
					'type'    => 'cookie'
				) );
			} else {
				array_push( $ppwp_cookie, (object) array(
					'prefix'  => 'contain',
					'content' => 'wp-postpass-role_',
					'type'    => 'cookie'
				) );
			}
			add_option( "WpFastestCacheExclude", json_encode( $ppwp_cookie ), null, "yes" );
			$fastest_cache->modify_htaccess_for_exclude();
		}
	}
}
