<?php
/**
 * Created by PhpStorm.
 * User: gaupoit
 * Date: 5/6/19
 * Time: 21:04
 */

if ( ! class_exists( 'WP_Protect_Password_Shared_Service' ) ) {

	/**
	 * Class WP_Protect_Password_Shared_Service
	 */
	class WP_Protect_Password_Shared_Service {

		/**
		 * Check post is required password for free version.
		 *
		 * @param int $post_id
		 *
		 * @return bool
		 */
		public function wpp_free_is_required_password( $post_id ) {
			$result    = $this->get_passwords( $post_id );
			$passwords = $result['passwords'];
			if ( ! $result['has_global_passwords'] && ! $result['has_role_passwords'] ) {
				return false;
			}

			return ! $this->is_valid_cookie( $post_id, $passwords );
		}

		/**
		 * Check whether the current cookie is valid.
		 *
		 * @param $post_id
		 * @param $passwords
		 *
		 * @return bool
		 */
		public function is_valid_cookie( $post_id, $passwords ) {
			if ( ! isset( $_COOKIE[ WP_Protect_Password_Constant::ROLE_POST_PASS_COOKIE . $post_id . COOKIEHASH ] ) ) {
				return false;
			}

			$cookie = sanitize_text_field( $_COOKIE[ WP_Protect_Password_Constant::ROLE_POST_PASS_COOKIE . $post_id . COOKIEHASH ] );
			$hash   = wp_unslash( $cookie );
			if ( 0 !== strpos( $hash, '$P$B' ) ) {
				return false;
			}

			$roles = $this->ppwp_get_current_role();
			foreach ( $passwords as $password ) {
				if ( wp_check_password( $password . $post_id, $hash ) ) {
					return true;
				}

				if ( false === $roles ) {
					continue;
				}

				foreach ( $roles as $role ) {
					if ( wp_check_password( $password . $role . $post_id, $hash ) ) {
						return true;
					}
				}
			}

			return false;
		}

		/**
		 * Get all free passwords
		 *
		 * @param $post_id
		 *
		 * @return array
		 */
		public function get_passwords( $post_id ) {
			// 1. Get all passwords.
			$global_passwords          = get_post_meta( $post_id, WP_Protect_Password_Constant::MULTIPLE_PASSWORDS_POST_META, true );
			$global_passwords          = ! empty( $global_passwords ) ? $global_passwords : array();
			$has_global_passwords      = ! empty( $global_passwords ) && is_array( $global_passwords );
			$meta_role                 = get_post_meta( $post_id, WP_Protect_Password_Constant::POST_PROTECTION_ROLES, true );
			$protected_roles           = false === unserialize( $meta_role ) ? array() : wp_unslash( unserialize( $meta_role ) );
			$filtered_protected_roles  = array_filter(
				$protected_roles,
				function ( $pass ) {
					if ( ! empty( $pass ) ) {
						return $pass;
					}
				}
			);
			$has_role_passwords        = ! empty( $filtered_protected_roles );
			$has_current_role_password = false;
			if ( $has_role_passwords ) {
				$roles = $this->ppwp_get_current_role();
				if ( false !== $roles ) {
					foreach ( $roles as $role ) {
						if ( array_key_exists( $role, $filtered_protected_roles ) ) {
							$has_current_role_password = true;
							array_push( $global_passwords, $protected_roles[ $role ] );
						}
					}
				}

			}

			$result = array(
				'passwords'                 => $global_passwords,
				'has_role_passwords'        => $has_role_passwords,
				'has_current_role_password' => $has_current_role_password,
				'has_global_passwords'      => $has_global_passwords,
			);

			return $result;
		}

		/**
		 * Get current role
		 *
		 * @return string|bool
		 */
		public function ppwp_get_current_role() {
			if ( ! is_user_logged_in() ) {
				return false;
			}

			$current_user = wp_get_current_user();
			if ( is_multisite() && is_super_admin( $current_user->ID ) ) {
				return array( 'administrator' );
			}

			return $current_user->roles;
		}
	}

}
