<?php

if ( ! class_exists( 'PPWP_Update_Service' ) ) {

	class PPWP_Update_Service {

		private $wpp_service_version;

		public function __construct() {
			$this->wpp_service_version = false === get_option( WP_Protect_Password_Constant::WPP_UPDATE_DEFAULT_SETTING ) ? '1.0' : get_option( WP_Protect_Password_Constant::WPP_UPDATE_DEFAULT_SETTING );
		}

		/**
		 * Update default setting
		 */
		public function update_default_setting() {
			if ( $this->wpp_service_version === '1.0' ) {
				$this->update_logic_password_entire_site();
				if ( ! wpp_is_gold_function() ) {
					$this->merge_data_for_free_when_update_new_version();
					migrate_default_password_to_ppwp();
				}
				$this->wpp_service_version = '1.1';
				update_option( WP_Protect_Password_Constant::WPP_UPDATE_DEFAULT_SETTING, $this->wpp_service_version );
			}
		}

		private function update_logic_password_entire_site() {
			$settings = get_option( WP_Protect_Password_Constant::GENERAL_SET_PASSWORD_OPTIONS );
			if ( $settings ) {
				$options = unserialize( $settings );
				if ( array_key_exists( "password_for_website", $options ) && ! empty( $options['password_for_website'] ) ) {
					$options['ppwp_apply_password_for_entire_site'] = "true";
					update_option( WP_Protect_Password_Constant::GENERAL_SET_PASSWORD_OPTIONS, serialize( $options ) );
				}
			}
		}

		/**
		 * Merge data for new version
		 */
		public function merge_data_for_free_when_update_new_version() {
			$global_password_meta = get_post_meta_by_meta_key( WP_Protect_Password_Constant::MULTIPLE_PASSWORDS_POST_META );
			$role_password_meta   = get_post_meta_by_meta_key( WP_Protect_Password_Constant::POST_PROTECTION_ROLES );
			foreach ( $global_password_meta as $global_password ) {
				$global_pass = unserialize( $global_password->meta_value );
				$global_pass = array_map( function ( $pass ) {
					return str_replace( array( ' ', '\\' ), array( '', '\\\\' ), $pass );
				}, $global_pass );
				$global_pass = array_unique( $global_pass );
				update_post_meta( $global_password->post_id, WP_Protect_Password_Constant::MULTIPLE_PASSWORDS_POST_META, $global_pass );
				$this->update_role_password( $role_password_meta, $global_pass, $global_password->post_id );
			}
		}

		/**
		 * Update role password for free when update new version
		 *
		 * @param $role_password_meta
		 * @param $global_pass
		 * @param $post_id
		 */
		private function update_role_password( $role_password_meta, $global_pass, $post_id ) {
			foreach ( $role_password_meta as $role ) {
				if ( $post_id === $role->post_id ) {
					$role_pass = @unserialize( @unserialize( $role->meta_value ) );
					if ( ! $role_pass ) {
						update_post_meta( $post_id, WP_Protect_Password_Constant::POST_PROTECTION_ROLES, serialize( array() ) );
					} else {
						$role_pass = array_map( function ( $pass ) {
							return str_replace( ' ', '', $pass );
						}, $role_pass );
						foreach ( $role_pass as $key => $value ) {
							if ( in_array( $value, $global_pass ) ) {
								$role_pass[ $key ] = '';
							}
						}
						update_post_meta( $post_id, WP_Protect_Password_Constant::POST_PROTECTION_ROLES, serialize( $role_pass ) );
					}
				}
			}
		}

	}

}
