<?php

add_meta_box(
	'ppwp_set_password_metabox',
	__( 'Password Protect WordPress', WP_Protect_Password_Constant::PPW_DOMAIN ),
	'ppwp_render_ui_set_password_meatbox',
	array( 'page', 'post' ),
	'side',
	'high'
);


/**
 * Function to render metabox set password for pages, posts, products in free version
 *
 * @param $post
 */
function ppwp_render_ui_set_password_meatbox( $post ) {
	$ppwp_admin      = new Wp_Protect_Password_Admin( '', '' );
	$protected_roles = ! get_post_meta( $post->ID, WP_Protect_Password_Constant::POST_PROTECTION_ROLES, true )
		? array() : wp_unslash( unserialize( get_post_meta( $post->ID, WP_Protect_Password_Constant::POST_PROTECTION_ROLES, true ) ) );

	$multiple_passwords = get_post_meta( $post->ID, WP_Protect_Password_Constant::MULTIPLE_PASSWORDS_POST_META, true );
	$password           = '';
	if ( ! empty( $multiple_passwords ) && is_array( $multiple_passwords ) ) {
		$password                  = esc_textarea( implode( "\n", $multiple_passwords ) );
		$global_password           = implode( ' ', $multiple_passwords );
		$protected_roles['global'] = $global_password;
	}

	$have_password_roles    = array_keys( array_filter( $protected_roles, array( $ppwp_admin, 'notEmptyValue' ) ) );
	$no_have_password_roles = sizeof( $have_password_roles );

	$no_roles = $no_have_password_roles > 1 ? $no_have_password_roles . ' roles' : $no_have_password_roles . ' role';

	$roles = get_editable_roles();

	?>
	<div id="passwords-roles-map" class="ppwp-post-protection">
            <span id="post-protection-status"> Password protected by
                <span id="number_roles"><?php echo esc_html( $no_roles ); ?></span>
            </span><a href="#protection" class="edit-post-protection hide-if-no-js pup-tooltip" role="button"
                      style="display: inline;">
			<span class="roles" aria-hidden="true">Edit</span>
		</a>
		<div style="display: none" id="post-protection">
			<p id="all_roles_select">
				<?php foreach ( $have_password_roles as $key => $role ): ?>
					<span><?php echo esc_html( $role ); ?></span>
				<?php endforeach; ?>
			</p>
			<div>
				<?php foreach ( $protected_roles as $key => $value ): ?>
					<span style="display: none"
					      id="post-protection-password-<?php echo esc_attr( $key ); ?>"><?php echo esc_html( $value ); ?></span>
				<?php endforeach; ?>
			</div>
			<div>
				<label for="post-protection-role">Role</label>
				<select class="pda-selected-role-select2" name="post-protection-role" id="is_role_selected">
					<option value="global">global</option>
					<?php foreach ( $roles as $role_name => $role_info ): ?>
						<option <?php echo esc_attr( in_array( $role_name, [] ) ? 'selected="selected"' : '' ); ?>" value="<?php echo esc_attr( $role_name ); ?>">
						<?php echo esc_html( $role_name ); ?>
						</option>
					<?php endforeach; ?>
				</select>
			</div>

			<!-- Region Data Save in HTML Tag START-->
			<?php foreach ( $roles as $role_name => $role_info ): ?>
				<?php $value = array_key_exists( $role_name, $protected_roles ) ? $protected_roles[ $role_name ] : ""; ?>
				<input type="hidden" id="<?php echo esc_attr( $role_name ); ?>"
				       name="data[<?php echo esc_attr( $role_name ); ?>]"
				       value="<?php echo esc_attr( $value ); ?>"/>
			<?php endforeach; ?>
			<!-- Region Data Save in HTML Tag END-->

			<div class="ppwp_wrap_role_password">
				<label id="label-password-post">Passwords</label>
				<input autocomplete="off" type="text" class="password post-protection-password" placeholder="Enter password"
				       name="post-protection-password"
				       id="post-protection-password"/>
				<textarea rows="3" id="ppwp_multiple_password" class="ppwp_multiple_password"
				          placeholder="<?php echo esc_attr( 'Enter one password per line', 'wp-protect-password' ); ?>"><?php echo esc_html( $password, 'wp-protect-password' ) ?></textarea>
			</div>
			<p class="ppwp-wrap-submit-hide">
				<input type="hidden" value="<?php echo esc_attr( $post->ID ); ?>" id="id_page_post">
				<button href="#protection" id="save_password" class="ppwp-button-submit">Submit</button>
				<a class="cancel-pda-protection-map button-cancel ppwp-button-hide">Hide</a>
			</p>
		</div>
	</div>
	<?php
}
