<?php
/**
 * Register sidebars.
 *
 * Registers our main widget area and the front page widget areas.
 *
 * @since qmp 1.0
 */

function themename_setup() {
	/*
	 * Let WordPress manage the document title.
	 * By adding theme support, we declare that this theme does not use a
	 * hard-coded <title> tag in the document head, and expect WordPress to
	 * provide it for us.
	 */
	add_theme_support('title-tag');

	// Add default posts and comments RSS feed links to head.
	add_theme_support( 'automatic-feed-links' );

	/*
	 * Enable support for Post Thumbnails on posts and pages.
	 *
	 * @link http://codex.wordpress.org/Function_Reference/add_theme_support#Post_Thumbnails
	 */
	add_theme_support( 'post-thumbnails' );

	/*
	 * Enable support for custom logo.
	 *
	 *  @since Twenty Sixteen 1.2
	 */
	add_theme_support( 'custom-logo', array(
		'height'      => 240,
		'width'       => 240,
		'flex-height' => true,
	) );

	add_theme_support( 'woocommerce' );

}
add_action('after_setup_theme','themename_setup');

/**
 * Custom template tags for this theme.
 */
require get_template_directory() . '/inc/template-tags.php';


/*Custom Theme Menu Locations*/
function register_my_menus() {
  register_nav_menus(
    array(
      'header-menu' => __( 'Header Menu' ),
      'footer-menu' => __( 'Footer Menu' )
    )
  );
}
add_action( 'init', 'register_my_menus' );



/*Custom Thumbnails For All Images*/
/*add_action( 'after_setup_theme', 'setup' );
function setup() {
    // ...
     
    add_theme_support( 'post-thumbnails' ); // This feature enables post-thumbnail support for a theme
    add_image_size( 'gall_thumb1', 253, 200, true );
    add_image_size( 'gall_thumb2', 200, 100, true );
     
    // ...
}*/

/*Custom Logo For Wordpress Login Page*/
function my_login_logo() { ?>
    <style type="text/css">
        #login h1 a, .login h1 a {
            background-image: url(<?php echo get_stylesheet_directory_uri(); ?>/images/admin-logo.png);
        }
    </style>
<?php }
add_action( 'login_enqueue_scripts', 'my_login_logo' );



function themename_widgets_init() {
	add_theme_support( 'post-thumbnails' );
	
	register_sidebar( array(
		'name' => __( 'Header Widget Area', 'themename' ),
		'id' => 'header-widget-area',
		'description' => __( 'Appears in the Header Area.', 'themename' ),
		'before_widget' => '<div id="%1$s" class="widget %2$s">',
		'after_widget' => '</div>',
		'before_title' => '<h3 class="widget-title">',
		'after_title' => '</h3>',
	) );

	
	

}

add_action( 'widgets_init', 'themename_widgets_init' );



function style_scripts()
{
	
	wp_enqueue_style('style',get_stylesheet_uri());
	wp_enqueue_style('slick_theme',get_template_directory_uri().'/css/slick-theme.css');
	wp_enqueue_style('slick_css',get_template_directory_uri().'/css/slick.css');
	wp_enqueue_style('bootstrap_style','https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/css/bootstrap.min.css');
	wp_enqueue_style('range_slider','https://cdnjs.cloudflare.com/ajax/libs/ion-rangeslider/2.3.0/css/ion.rangeSlider.min.css');
	wp_enqueue_style('owl_css',get_template_directory_uri().'/owls/owl.carousel.min.css');
    wp_enqueue_style('owl_theme',get_template_directory_uri().'/owls/owl.theme.default.min.css');

	wp_enqueue_script('main_script','https://ajax.googleapis.com/ajax/libs/jquery/3.4.0/jquery.min.js', array( 'jquery' ), '1.0.0', true);
 	wp_enqueue_script('popper_script','https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.7/umd/popper.min.js', array( 'jquery' ), '1.0.0', true);
 	wp_enqueue_script('bootstrap_script','https://maxcdn.bootstrapcdn.com/bootstrap/4.3.1/js/bootstrap.min.js', array( 'jquery' ), '1.0.0', true);
 	wp_enqueue_script('slick_script',get_template_directory_uri().'/js/slick.min.js', array( 'jquery' ), '1.0.0', true);
 	wp_enqueue_script('range_js','https://cdnjs.cloudflare.com/ajax/libs/ion-rangeslider/2.3.0/js/ion.rangeSlider.min.js', array( 'jquery' ), '1.0.0', true);
 	wp_enqueue_script('owl_js',get_template_directory_uri().'/owls/owl.carousel.js', array( 'jquery' ), '1.0.0', true);

	
}
add_action('wp_enqueue_scripts','style_scripts');




function custom_excerpt_length($length)
	{
		return 20;		
	}
	add_filter('excerpt_length','custom_excerpt_length',999);
	
function new_excerpt_more($more)
	{
		global $post;
		return '<a class="moretag" href="'.get_permalink($post->ID).'">READ MORE</a>';
	}
	add_filter('excerpt_more','new_excerpt_more');

function my_search_form( $form ) {
    $form = '<form role="search" method="get" class="searchform" action="' . home_url( '/' ) . '" >
    <div><label class="screen-reader-text" for="s">' . __( '' ) . '</label>
    <input type="text" value="' . get_search_query() . '" name="s" id="s" placeholder="search this website..." />
    <input type="submit" id="searchsubmit" class="search-btn" value="" />
    </div>
    </form>';

    return $form;
}

add_filter( 'get_search_form', 'my_search_form' );

/*========== To Remove Front End Display Sec From All Pages ================*/
add_filter('types_information_table', '__return_false');


if( function_exists('acf_add_options_page') ) {
	
	acf_add_options_page();
	
}

if( function_exists('acf_set_options_page_title') )
{
    acf_set_options_page_title( __('Footer Options') );
}

add_action("wp_ajax_nopriv_getProduct","getProduct");
add_action("wp_ajax_getProduct","getProduct");

function getProduct()
{
global $wpdb;
$prod_category=$_GET['prod_category'];
$business_stage = $_GET['business_stage'];

if($prod_category){ $explodeCat = explode(', ',$prod_category);
$ar = array(
                    'taxonomy' => 'product_item_category',
                    'field' => 'name',
                    'terms'  => $explodeCat,
                    'operator' => 'IN'
                );
}
//echo json_encode($business_stage);die;

$postArg = array('post_type'=>'product_item','posts_per_page'=>-1,'post_status'=>'publish','order'=>'desc','meta_key' => 'business_stage','meta_value' => $business_stage,
          'tax_query' => array(
            $ar

          ));
        //print_r($prod_category);die;
        $getPost = new wp_query($postArg);
        global $post;
        $response=array();
        if($getPost->have_posts()){
            
            $i=0;
            while ( $getPost->have_posts()):$getPost->the_post();
                $response[$i]['product_main_color'] = get_field('product_main_color');
                $response[$i]['product_icon'] =  get_field('product_icon');
                $response[$i]['title'] =  $post->post_title;
                $response[$i]['content'] = get_the_content();
                $response[$i]['product_price'] = get_field('product_price');
                $response[$i]['product_price_tax_instruction'] = get_field('product_price_tax_instruction');
                $response[$i]['learn_more_button_color'] = get_field('learn_more_button_color');
                $response[$i]['product_learn_more_link'] = get_field('product_learn_more_link');
                $response[$i]['select_button_color'] = get_field('select_button_color');
                $response[$i]['product_button_link'] = get_field('product_button_link');
                $i++;
            endwhile;
        }

echo json_encode($response);die;
}



add_action("wp_ajax_nopriv_getProductSearch","getProductSearch");
add_action("wp_ajax_getProductSearch","getProductSearch");

function getProductSearch()
{
global $wpdb;
$prod_search=$_GET['prod_search'];


$postArg = array('post_type'=>'product_item','posts_per_page'=>-1,'post_status'=>'publish','order'=>'desc', "s" => $prod_search);

        $getPost = new wp_query($postArg);
        global $post;
        $response=array();
        if($getPost->have_posts()){
            
            $i=0;
            while ( $getPost->have_posts()):$getPost->the_post();
                $response[$i]['product_main_color'] = get_field('product_main_color');
                $response[$i]['product_icon'] =  get_field('product_icon');
                $response[$i]['title'] =  $post->post_title;
                $response[$i]['content'] = get_the_content();
                $response[$i]['product_price'] = get_field('product_price');
                $response[$i]['product_price_tax_instruction'] = get_field('product_price_tax_instruction');
                $response[$i]['learn_more_button_color'] = get_field('learn_more_button_color');
                $response[$i]['product_learn_more_link'] = get_field('product_learn_more_link');
                $response[$i]['select_button_color'] = get_field('select_button_color');
                $response[$i]['product_button_link'] = get_field('product_button_link');
                $i++;
            endwhile;
        }

echo json_encode($response);die;
}

