<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       www.buildwps.com/prevent-direct-access-premium-plugin/?utm_source=user-website&utm_medium=pluginpage&utm_campaign=plugin-author-link
 * @since      1.0.0
 *
 * @package    Wp_Protect_Password
 * @subpackage Wp_Protect_Password/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Wp_Protect_Password
 * @subpackage Wp_Protect_Password/admin
 * @author     BWPS <hello@ymese.com>
 */
class Wp_Protect_Password_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 *
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wp_Protect_Password_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wp_Protect_Password_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		$screens = [ 'edit-post', 'edit-page', 'post', 'page', 'edit-product', 'product' ];
		if ( in_array( get_current_screen()->id, $screens ) ) {
			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/wp-protect-password-admin.css', array(), $this->version, 'all' );
			if ( ! is_gold_function() ) {
				wp_enqueue_style( 'load-css-for-free-version', plugin_dir_url( __FILE__ ) . 'css/wp-protect-password-free.css', array(), $this->version, 'all' );
			}
		}

		$screens_for_toastr = [ "toplevel_page_wp_protect_password_options", "post", "page", "product" ];
		if ( in_array( get_current_screen()->id, $screens_for_toastr ) ) {
			wp_enqueue_style( $this->plugin_name . '-toastr', 'https://cdnjs.cloudflare.com/ajax/libs/toastr.js/latest/toastr.min.css' );
		}

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Wp_Protect_Password_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Wp_Protect_Password_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		if ( get_current_screen()->id === "toplevel_page_wp_protect_password_options" ) {
			wp_enqueue_script( $this->plugin_name . '-gold-license', plugin_dir_url( __FILE__ ) . './js/wp-protect-password-license.js', array( 'jquery' ), $this->version );
			wp_localize_script( $this->plugin_name . '-gold-license', 'wp_protect_password_gold_license_data', array(
				'ajax_url' => admin_url( 'admin-ajax.php' ),
			) );
		}

		$free_screens = [ 'post', 'page', 'product' ];
		if ( in_array( get_current_screen()->id, $free_screens ) ) {

			wp_enqueue_script( $this->plugin_name . '-save_password_js' . 'publish-box', plugin_dir_url( __FILE__ ) . 'js/wp-protect-password-publish-box.js', array(), $this->version, 'all' );
			wp_localize_script( $this->plugin_name . '-save_password_js' . 'publish-box', 'save_password_data', array(
				'ajax_url' => admin_url( 'admin-ajax.php' ),
			) );
		}

		$screens_for_toastr = [ "toplevel_page_wp_protect_password_options", "post", "page", "product" ];
		if ( in_array( get_current_screen()->id, $screens_for_toastr ) ) {
			wp_enqueue_script( $this->plugin_name . '-toastr', 'https://cdnjs.cloudflare.com/ajax/libs/toastr.js/latest/toastr.min.js' );
		}

		if ( "toplevel_page_wp_protect_password_options" === get_current_screen()->id ) {
			wp_enqueue_script( 'select2_js' . $this->plugin_name, plugin_dir_url( WPPP_BASE_FILE ) . 'admin/js/lib/select2.min.js', array( 'jquery' ), '4.0.3', true );
		}

		if ( ! is_gold_function() && 'plugins' === get_current_screen()->id ) {
			wp_enqueue_script( $this->plugin_name . '-handle-deactivate', plugin_dir_url( __FILE__ ) . 'js/wp-protect-password-handle-deactivate.js', array(), $this->version, 'all' );
			wp_localize_script( $this->plugin_name . '-handle-deactivate', 'data-handle-deactivate', array(
				'ajax_url' => admin_url( 'admin-ajax.php' ),
			) );
		}

	}

	public function pup_post_pass() {
		$password = wp_unslash( $_POST['post_password'] );
		$post_id  = wp_unslash( $_POST['post_id'] );

		if ( is_gold_function() ) {
			$user = wp_get_current_user();
			$role = "";
			if ( $user->ID !== 0 ) {
				$role = $user->roles[0];
			}
			$is_valid = $this->check_password_gold_version( $password, $post_id, $role );

			if ( $is_valid ) {

				$ppwp_auto_protect_all_child_pages = wpp_get_settings_boolean( WP_Protect_Password_Constant::WPP_AUTO_PROTECT_ALL_CHILD_PAGES );
				if ( $ppwp_auto_protect_all_child_pages ) {
					$parents   = get_post_ancestors( $post_id );
					$parent_id = $parents ? $parents[ count( $parents ) - 1 ] : $post_id;
				} else {
					$parent_id = $post_id;
				}

				$this->setPasswordToCookie( $password . $role . $parent_id, WP_Protect_Password_Constant::GOLD_PASS_COOKIE . $parent_id );
				$url_redirect = preg_replace( '/[&?]ppwp_enter_wrong_password=true/', '', wp_get_referer() );
				wp_safe_redirect( $url_redirect );
				exit();
			} else {
				$query_str = parse_url( wp_get_referer(), PHP_URL_QUERY );
				parse_str( $query_str, $query_params );
				if ( array_key_exists( 'ppwp_enter_wrong_password', $query_params ) && $query_params['ppwp_enter_wrong_password'] === "true" ) {
					wp_safe_redirect( wp_get_referer() );
				} else {
					if ( empty( $query_params ) ) {
						wp_safe_redirect( wp_get_referer() . "?ppwp_enter_wrong_password=true" );
					} else {
						wp_safe_redirect( wp_get_referer() . "&ppwp_enter_wrong_password=true" );
					}
				}
				exit();
			}
		} else {
			$is_valid_free = $this->check_password_free_version( $password, $post_id );
			wp_safe_redirect( wp_get_referer() );
			exit();
		}
	}

	public function check_password_free_version( $password, $post_id ) {
		if ( $this->ppwp_check_global_password( $post_id, $password ) ) {
			$this->setPasswordToCookie( $password . $post_id, WP_Protect_Password_Constant::ROLE_POST_PASS_COOKIE . $post_id );
			return true;
		} else {
			if ( is_user_logged_in() ) {
				$role_meta = get_post_meta( $post_id, WP_Protect_Password_Constant::POST_PROTECTION_ROLES, true );
				if ( $role_meta ) {
					$protected_roles = wp_unslash( unserialize( $role_meta ) );
					$user            = wp_get_current_user();
					if ( is_multisite() && is_super_admin( $user->ID ) ) {
						$roles = array( 'administrator' );
					} else {
						$roles = $user->roles;
					}

					return $this->free_is_password_validate( $roles, $protected_roles, $password, $post_id );
				}

			}
		}

		return false;
	}

	private function setPasswordToCookie( $password, $cookie_name ) {
		$password_hashed         = wp_hash_password( $password );
		$expire                  = apply_filters( 'post_password_expires', time() + 7 * DAY_IN_SECONDS );
		$password_cookie_expired = wpp_get_settings_value( WP_Protect_Password_Constant::WPP_PASSWORD_COOKIE_EXPIRED );
		if ( ! empty( $password_cookie_expired ) ) {
			$time = explode( " ", $password_cookie_expired )[0];
			$unit = $this->get_unit_time( $password_cookie_expired );
			if ( $unit !== 0 ) {
				$expire = apply_filters( 'post_password_expires', time() + (int) $time * $unit );
			}
		}

		$referer = wp_get_referer();
		if ( $referer ) {
			$secure = ( 'https' === parse_url( $referer, PHP_URL_SCHEME ) );
		} else {
			$secure = false;
		}
		setcookie( $cookie_name . COOKIEHASH, $password_hashed, $expire, COOKIEPATH, COOKIE_DOMAIN, $secure );
	}

	private function free_is_password_validate( $userRoles, $protectedRoles, $password, $post_id ) {
		$isValid = false;
		foreach ( $userRoles as $role ) {
			if ( array_key_exists( $role, $protectedRoles ) ) {
				if ( $protectedRoles[ $role ] === $password ) {
					$this->setPasswordToCookie( $password . $role . $post_id, WP_Protect_Password_Constant::ROLE_POST_PASS_COOKIE . $post_id );
					$isValid = true;
					break;
				}
			}
		}

		return $isValid;
	}

	function ppwp_check_global_password( $post_id, $password ) {
		$multiple_passwords = get_post_meta( $post_id, WP_Protect_Password_Constant::MULTIPLE_PASSWORDS_POST_META, true );
		if ( ! empty( $multiple_passwords ) && is_array( $multiple_passwords ) ) {
			foreach ( $multiple_passwords as $pass ) {
				if ( $password === $pass ) {
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * Check whether password is valid.
	 *
	 * @param $password
	 * @param $post_id
	 *
	 * @return bool Success
	 */
	private function check_password_gold_version( $password, $post_id, $role ) {
		$service = new WP_Protect_Password_Service();

		return $service->is_valid_password( $post_id, $password, $role );
	}

	public function notEmptyValue( $val ) {
		return ! empty( $val );
	}

	function wp_protect_password_create_plugin_menu() {
		require_once plugin_dir_path( dirname( __FILE__ ) ) . '/includes/class-wp-protect-password-settings.php';
		$setting_page   = new Wp_Protect_Password_Settings();
		$menu_page_name = add_menu_page( 'Protect Password Settings', 'Password Protect WordPress', 'administrator', 'wp_protect_password_options', array(
			$setting_page,
			'wp_protect_password_render_ui'
		), 'dashicons-unlock' );

		add_action( 'admin_print_styles-' . $menu_page_name, array( $this, 'wp_protect_password_setting_assets' ) );
	}

	public function wp_protect_password_setting_assets() {

		wp_enqueue_script( 'wp_protect_password_js' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './js/wp-protect-password-setting-general.js', array( 'jquery' ), $this->version );
		wp_localize_script( 'wp_protect_password_js' . $this->plugin_name, 'wp_protect_password_data', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );

		wp_register_style( 'wp_protect_password_setting_' . $this->plugin_name, plugin_dir_url( __FILE__ ) . './css/wp-protect-password-setting.css', array(), $this->version, 'all' );
		wp_enqueue_style( 'wp_protect_password_setting_' . $this->plugin_name );
	}

	public function wp_protect_password_update_password() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, WP_Protect_Password_Constant::GENERAL_PASSWORD_FORM_NONCE ) ) {
			wp_die( 'Invalid nonce' );
		}

		$data_settings = $_REQUEST['settings'];
		if ( array_key_exists( "ppwp_apply_password_for_entire_site", $data_settings ) && $data_settings["ppwp_apply_password_for_entire_site"] === "true" ) {
			if ( ! array_key_exists( "ppwp_set_new_password_for_entire_site", $data_settings ) ) {
				$data_settings["password_for_website"] = md5( $data_settings["password_for_website"] );
				update_option( WP_Protect_Password_Constant::GENERAL_SET_PASSWORD_OPTIONS, serialize( $data_settings ) );

				// Clear cache
				ppwp_clear_cache();
			} else if ( array_key_exists( "ppwp_set_new_password_for_entire_site", $data_settings ) && $data_settings["ppwp_set_new_password_for_entire_site"] === "true" ) {
				$data_settings["password_for_website"] = md5( $data_settings["password_for_website"] );
				unset( $data_settings['ppwp_set_new_password_for_entire_site'] );
				update_option( WP_Protect_Password_Constant::GENERAL_SET_PASSWORD_OPTIONS, serialize( $data_settings ) );

				// Clear cache
				ppwp_clear_cache();
			}
		} else {
			$data_settings["password_for_website"] = "";
			update_option( WP_Protect_Password_Constant::GENERAL_SET_PASSWORD_OPTIONS, serialize( $data_settings ) );

			// Clear cache
			ppwp_clear_cache();
		}
	}

	public function wp_protect_password_update_settings() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, WP_Protect_Password_Constant::GENERAL_SETTING_FORM_NONCE ) ) {
			wp_die( 'Invalid nonce' );
		}
		$data_settings = $_REQUEST['settings'];

		if ( is_gold_function() ) {
			if ( $data_settings['wpp_apply_password_for_pages_posts'] === "true" ) {
				$ppwp_repo = new WP_Protect_Password_Repo( get_table_name() );
				foreach ( $data_settings['wpp_pages_posts_select'] as $post_id ) {
					if ( ! is_null( $ppwp_repo->get_password_by_password_post_id_and_not_type( $data_settings['wpp_password_for_pages_posts'], $post_id, WP_Protect_Password_Constant::CAMPAIGN_TYPE['COMMON'] ) ) ) {
						wp_send_json( array(
							'is_error' => true,
							'message'  => __( "The password is already in use. Please create a new one.", "prevent-direct-access-gold" )
						) );
						wp_die();
					}
				}
				$ppwp_service = new WP_Protect_Password_Service();
				$ppwp_service->insert_password_for_pages_posts( $data_settings['wpp_pages_posts_select'], wp_unslash( $data_settings['wpp_password_for_pages_posts'] ) );
			}
			if ( $data_settings['wpp_apply_password_for_pages_posts'] === "false" ) {
				delete_all_password_type_is_common();
			}
		}
		update_option( WP_Protect_Password_Constant::GENERAL_SETTING_OPTIONS, wp_json_encode( $data_settings ) );
	}

	public function parse_auth_cookie() {
		$cookie_name = WP_Protect_Password_Constant::COOKIE_NAME_WP_PROTECT_PASSWORD;
		if ( empty( $_COOKIE[ $cookie_name ] ) ) {
			return false;
		}

		$cookie = $_COOKIE[ $cookie_name ];

		$cookie_elements = explode( '|', $cookie );

		if ( count( $cookie_elements ) != 2 ) {
			return false;
		}

		return $cookie_elements;
	}

	public function validate_auth_cookie() {
		if ( ! $cookie_elements = $this->parse_auth_cookie() ) {
			return false;
		}

		if ( (int) $cookie_elements[1] < time() ) {
			return false;
		}

		$password_for_website = "";
		$settings             = get_option( WP_Protect_Password_Constant::GENERAL_SET_PASSWORD_OPTIONS );
		if ( $settings ) {
			$options = unserialize( $settings );
			if ( array_key_exists( "password_for_website", $options ) ) {
				$password_for_website = $options["password_for_website"];
			}
		}

		$hash = hash_hmac( 'md5', WP_Protect_Password_Constant::COOKIE_NAME_WP_PROTECT_PASSWORD, $password_for_website );
		if ( $cookie_elements[0] != $hash ) {
			return false;
		}

		return true;
	}

	function render_form_enter_password() {
		if ( $this->validate_auth_cookie() ) {
			return;
		}

		$settings = get_option( WP_Protect_Password_Constant::GENERAL_SET_PASSWORD_OPTIONS );
		if ( $settings ) {
			$options = unserialize( $settings );
			if ( array_key_exists( "password_for_website", $options ) ) {
				$password_for_website = $options["password_for_website"];
			}
			if ( isset( $password_for_website ) && $password_for_website != "" ) {
				if ( isset( $_REQUEST['input_wp_protect_password'] ) ) {
					$password = $_REQUEST['input_wp_protect_password'];
					if ( md5( $password ) == $password_for_website ) {
						$expiration              = time() + 7 * DAY_IN_SECONDS;
						$password_cookie_expired = wpp_get_settings_value( WP_Protect_Password_Constant::WPP_PASSWORD_COOKIE_EXPIRED );

						if ( ! empty( $password_cookie_expired ) ) {
							$time = explode( " ", $password_cookie_expired )[0];
							$unit = $this->get_unit_time( $password_cookie_expired );
							if ( $unit !== 0 ) {
								$expiration = time() + (int) $time * $unit;
							}
						}

						$hash   = hash_hmac( 'md5', WP_Protect_Password_Constant::COOKIE_NAME_WP_PROTECT_PASSWORD, $password_for_website );
						$cookie = $hash . "|" . $expiration;
						setcookie( WP_Protect_Password_Constant::COOKIE_NAME_WP_PROTECT_PASSWORD, $cookie, $expiration, COOKIEPATH, COOKIE_DOMAIN );

						global $wp;
						$current_url = add_query_arg( $wp->query_string, '', home_url( $wp->request ) );
						wp_redirect( $current_url );
						ppwp_clear_cache();
						die;
					}
				}
				?>
				<div class="pda-form-login">
					<h1>
						<a></a>
					</h1>
					<form action="<?php home_url(); ?>" method="post">
						<label for="">Password</label>
						<input class="input_wp_protect_password" type="password" id="input_wp_protect_password"
						       name="input_wp_protect_password">
						<input type="submit" class="button button-primary button-login" value="Login">
					</form>
				</div>
				<style type="text/css">
					body {
						background: #f1f1f1;
						min-width: 0;
						font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Oxygen-Sans, Ubuntu, Cantarell, "Helvetica Neue", sans-serif;
					}

					.pda-form-login h1 {
						margin: 0;
					}

					.pda-form-login {
						width: 320px;
						padding: 8% 0 0;
						margin: auto;
					}

					.pda-form-login h1 a {
						background-image: none, url("<?php echo plugin_dir_url( __FILE__ ) ?>images/password-protect-square-logo.png");
						background-size: 84px;
						background-position: center top;
						background-repeat: no-repeat;
						color: #444;
						height: 84px;
						font-size: 20px;
						line-height: 1.3em;
						margin: 0 auto 25px;
						padding: 0;
						width: 84px;
						text-indent: -9999px;
						outline: 0;
						display: block;
					}

					.pda-form-login form {
						margin-top: 20px;
						margin-left: 0;
						padding: 26px 24px 46px;
						background: #fff;
						box-shadow: 0 1px 3px rgba(0, 0, 0, .13);
					}

					.pda-form-login .button-login {
						height: 30px;
						line-height: 28px;
						padding: 0 12px 2px;
						vertical-align: baseline;
						float: right;
						background: #0085ba;
						border-color: #0073aa #006799 #006799;
						box-shadow: 0 1px 0 #006799;
						color: #fff;
						text-decoration: none;
						text-shadow: 0 -1px 1px #006799, 1px 0 1px #006799, 0 1px 1px #006799, -1px 0 1px #006799;
						display: inline-block;
						font-size: 13px;
						margin: 0;
						cursor: pointer;
						border-width: 1px;
						border-style: solid;
						border-radius: 3px;
						white-space: nowrap;
						box-sizing: border-box;
						font-family: inherit;
						font-weight: inherit;
					}

					.pda-form-login label {
						color: #72777c;
						font-size: 14px;
					}

					.pda-form-login .input_wp_protect_password {
						background: #fbfbfb;
						font-size: 24px;
						width: 100%;
						padding: 3px;
						margin: 2px 6px 16px 0;
						border: 1px solid #ddd;
						box-shadow: inset 0 1px 2px rgba(0, 0, 0, .07);
						background-color: #fff;
						color: #32373c;
						outline: 0;
						transition: 50ms border-color ease-in-out;
					}
				</style>
				<?php
				die;
			}
		}
	}

	public function wp_protect_password_save_password() {
		$data_settings          = $_REQUEST["settings"];
		$password               = $data_settings["save_password"];
		$id                     = $data_settings["id_page_post"];
		$role                   = $data_settings["is_role_selected"];
		$ppwp_multiple_password = $data_settings["ppwp_multiple_password"];

		$post_meta       = get_post_meta( $id, WP_Protect_Password_Constant::POST_PROTECTION_ROLES, true );
		$global_password = get_post_meta( $id, WP_Protect_Password_Constant::MULTIPLE_PASSWORDS_POST_META, true );
		$current         = ( ! empty( $post_meta ) && false !== @unserialize( $post_meta ) ) ? @unserialize( $post_meta ) : array();

		if ( $role === 'global' ) {
			// Validate for global
			free_validate_password_type_is_global( $ppwp_multiple_password, $global_password, $current );
			$multiple_passwords = array_filter( array_map( 'trim', explode( "\n", $ppwp_multiple_password ) ) );
			update_post_meta( $id, WP_Protect_Password_Constant::MULTIPLE_PASSWORDS_POST_META, $multiple_passwords );

			// Add page or post to list not cache
			free_add_page_or_post_to_list_not_cache_with_type_is_global( $multiple_passwords, $id, $current );
			$current[ $role ] = implode( "\n", $multiple_passwords );
		} else {
			// Validate for role
			free_validate_password_type_is_role( $role, $password, $global_password, $current );
			$current[ $role ] = $password;
			// TODO: Need to check when not use update_post_meta
			delete_post_meta( $id, WP_Protect_Password_Constant::POST_PROTECTION_ROLES );
			add_post_meta( $id, WP_Protect_Password_Constant::POST_PROTECTION_ROLES, wp_slash( serialize( $current ) ) );

			// Add page or post to list not cache
			free_add_page_or_post_to_list_not_cache_with_type_is_role( $password, $id, $current );
			if ( ! empty( $global_password ) ) {
				$current['global'] = implode( "\n", $global_password );
			}
		}

		wp_send_json( $current );
		wp_die();
	}

	function wp_protect_password_gold_check_license() {
		$nonce = $_REQUEST['security_check'];
		if ( ! wp_verify_nonce( $nonce, WP_Protect_Password_Constant::LICENSE_FORM_NONCE ) ) {
			error_log( 'not verify nonce', 0 );
			wp_die( 'invalid_nonce' );
		}
		$license = $_REQUEST['license'];

		if ( empty( $license ) ) {
			update_option( WP_Protect_Password_Constant::LICENSE_OPTIONS, false );
			update_option( WP_Protect_Password_Constant::LICENSE_ERROR, 'Invalid license' );
			update_option( WP_Protect_Password_Constant::LICENSE_KEY, '' );
			wp_send_json( false );
		}

		$result = YME_LICENSE::checkLicense( $license, 'wpp', get_site_option( WP_Protect_Password_Constant::APP_ID ) );

		$data = $result['data'];

		if ( ! $data ) {
			$result['data']['errorMessage'] = "There is something's wrong. Please <a href=\"hello@preventdirectaccess.com\">contact</a> the plugin owner!";
			update_option( WP_Protect_Password_Constant::LICENSE_OPTIONS, false );
			update_option( WP_Protect_Password_Constant::LICENSE_ERROR, $data['errorMessage'] );
			update_option( WP_Protect_Password_Constant::LICENSE_KEY, '' );
		} else if ( is_object( $data ) && property_exists( $data, 'errorMessage' ) ) {
			update_option( WP_Protect_Password_Constant::LICENSE_OPTIONS, false );
			update_option( WP_Protect_Password_Constant::LICENSE_ERROR, $data->errorMessage );
			update_option( WP_Protect_Password_Constant::LICENSE_KEY, '' );
		} else {
			update_option( WP_Protect_Password_Constant::LICENSE_KEY, $license );
			update_option( WP_Protect_Password_Constant::LICENSE_OPTIONS, true );
			update_option( WP_Protect_Password_Constant::LICENSE_ERROR, '' );
		}
		wp_send_json( $result );
		wp_die();
	}

	function save_product_id_to_option() {
		if ( get_site_option( WP_Protect_Password_Constant::APP_ID ) === false ) {
			$configs = include( PDA_PWD_DIR_PATH . 'includes/class-wp-protect-password-configs.php' );
			update_site_option( WP_Protect_Password_Constant::APP_ID, $configs->app_id );
		}
	}

	public function wppp_rest_api_init_cb() {
		$api = new WP_Protect_Password_Api;
		$api->register_rest_routes();
	}

	public function handle_plugin_update() {
		install_pda_pwd_table();
	}

	public function get_unit_time( $password_cookie_expired ) {
		$time_die = explode( " ", $password_cookie_expired );
		$unit     = 0;
		if ( count( $time_die ) === 2 ) {
			if ( $time_die[1] === "minutes" ) {
				$unit = 60;
			} else if ( $time_die[1] === "hours" ) {
				$unit = 3600;
			} else if ( $time_die[1] === "days" ) {
				$unit = 86400;
			}
		}

		return $unit;
	}

	public function handle_db_update() {
		install_pda_pwd_table();
	}

	public function add_custom_metabox_to_edit_page_for_free_version() {
		include PDA_PWD_DIR_PATH . '/includes/views/view-wp-protect-password-metabox-free.php';
	}

	/**
	 * Function handle update default setting and plugin update
	 */
	public function handle_update_for_plugin() {
		$wpp_update_service = new PPWP_Update_Service();
		$wpp_update_service->update_default_setting();
	}

}
